#include "splash.h"

#include "config.h"

#include <qpixmap.h>

Splash::Splash(QWidget *parent, const char *name)
: QWidget(parent, name)
{
	setBackgroundColor(QColor(0, 0, 0));
	setBackgroundPixmap(QPixmap(DATADIR "/pics/splash.png"));

	//setMinimumSize(400, 400);
	setFixedSize(autowidth(), autoheight());
}

Splash::~Splash()
{
}

int Splash::autowidth()
{
	return 400;
}

int Splash::autoheight()
{
	return 400;
}

