#ifndef _player_h
#define _player_h

#include "sprite.h"

class Player : public Sprite
{
public:
	Player(int id, int x, int y, QCanvasSprite* sprite);

	void numLives(int);
	int numLives(void);
	void numBombs(int);
	int numBombs(void);
	void power(int);
	int power(void);
	void numFlash(int);
	int numFlash(void);
	void move(int x, int y);
	void setFd(int fd);
	int fd();
	void steps(int);
	int steps(void);
	void totalsteps(int);
	int totalsteps(void);
	void stepcounter(int);
	int stepcounter(void);
	void visibility(int v);
	int visibility(void);
	void setSeat(int seat);
	int seat();

private:
	int m_lives;
	int m_bombs;
	int m_power;
	int m_fd;
	int m_steps;
	int m_totalsteps;
	int m_stepcounter;
	int m_visibility;
	int m_flash;

	int m_leftDirection;
	int m_rightDirection;
	int m_upDirection;
	int m_downDirection;

	int m_seat;
};

#endif

