#ifndef KAMIKAZE_BLAST_H
#define KAMIKAZE_BLAST_H

#include "sprite.h"

class Blast : public Sprite
{
public:
	Blast(int id, int x, int y, QCanvasSprite* sprite);

	void proceed(void);
	int step();
};

#endif

