#ifndef PROTOCOL_H
#define PROTOCOL_H

/*enum Items
{
	item_bomb = 1,
	item_life = 2
};

enum Actions
{
	action_bomb_placed = 1,
	action_bomb_exploded = 2,
	action_player_move = 3,
	action_player_damage = 4,
	action_item_erase = 5,
	action_ground_damage = 6
};*/

enum Actions
{
	action_newplayer,
	action_newmonster,
	action_newitem,
	action_newexit,
	action_newblast,
	action_newbomb,
	action_item,
	action_removeplayer,
	action_removemonster,
	action_removeitem,
	action_removebomb,
	action_removeblast,
	action_activateitem,
	action_activateexit,
	action_highlightitem,
	action_highlightexit
};

enum MapCodes
{
	map_select,
	map_selected
};

enum StateCodes
{
	state_start,
	state_stop
};

enum MoveDirections
{
	move_up,
	move_down,
	move_left,
	move_right
};
enum MoveEntitites
{
	move_player,
	move_monster
};

enum Opcodes
{
	opcode_cheat,
	opcode_map,
	opcode_rules,
	opcode_state,
	opcode_move,
	opcode_action,
	opcode_over
};

#endif

