#ifndef KNET_H
#define KNET_H

#include "mnet.h"

class Rule;

class KNet : public MNet
{
	public:
		KNet();
		~KNet();

		KNet &operator<<(Rule *rule);
		KNet &operator>>(Rule **rule);

		KNet &operator<<(Net::SpecialOpcodes value){MNet::operator<<(value); return *this;}
		KNet &operator<<(MNet::ChannelOpcodes value){MNet::operator<<(value); return *this;}
		KNet &operator<<(MNet::ModeOpcodes value){MNet::operator<<(value); return *this;}

		KNet &operator<<(int value){MNet::operator<<(value); return *this;}
		KNet &operator<<(char *value){MNet::operator<<(value); return *this;}

		KNet &operator>>(int *value){Net::operator>>(value); return *this;}
		KNet &operator>>(char **value){Net::operator>>(value); return *this;}
};

#endif

