#include "rule.h"

Rule::Rule()
{
	m_custom = 0;

	player_lives = 1;
	player_power = 1;
	player_bombs = 3;
	monster_lives = 1;
	monster_power = 1;
	monster_bombs = 1;
	monster_blastfrequency = 20;
	flash_number = 1;
	invisible_number = 1;
	grail_number = 1;
	speed_number = 3;
	power_number = 3;
	life_number = 3;
	bomb_number = 3;
	bomb_destroy = 1;
	bomb_lock = 1;
	bomb_duration = 10;
	timebomb_allowed = 1;
	moveablebomb_allowed = 1;
	scree_percentage = 25;
	wall_damage = 0;
	ground_damage = 0;
}

void Rule::setCustom()
{
	m_custom = 1;
}

int Rule::custom()
{
	return m_custom;
}

