#ifndef RULE_H
#define RULE_H

class Rule
{
	public:
		Rule();

		int custom();
		void setCustom();

		int player_lives;
		int player_power;
		int player_bombs;
		int monster_lives;
		int monster_power;
		int monster_bombs;
		int monster_blastfrequency;
		int flash_number;
		int invisible_number;
		int grail_number;
		int speed_number;
		int power_number;
		int life_number;
		int bomb_number;
		int bomb_destroy;
		int bomb_lock;
		int bomb_duration;
		int timebomb_allowed;
		int moveablebomb_allowed;
		int scree_percentage;
		int wall_damage;
		int ground_damage;

	private:
		int m_custom;
};

#endif

