#include <kapplication.h>
#include <kaboutdata.h>
#include <klocale.h>
#include <kcmdlineargs.h>
#include <kdebug.h>

#include "config.h"
#include "toplevel.h"

int main(int argc, char **argv)
{
	KApplication *app;
	KAboutData *about;

	about = new KAboutData("kamikazeeditor",
		I18N_NOOP("Kamikaze Editor"),
		VERSION,
		I18N_NOOP("Level editor for the Kamikaze game"),
		KAboutData::License_GPL_V2,
		"Copyright (C) 2003 - 2006 Josef Spillner",
		I18N_NOOP("Kamikaze is a not so harmless action game for 1-8 players."),
		"http://kamikaze.coolprojects.org/",
		"spillner@kde.org");
	about->addAuthor("Josef Spillner", I18N_NOOP("Maintainer"), "spillner@kde.org");
	about->setTranslator(I18N_NOOP("TRANSLATOR-NAME"), I18N_NOOP("TRANSLATOR-EMAIL"));

	KLocale::setMainCatalogue("kamikaze");
	KCmdLineArgs::init(argc, argv, about);

	app = new KApplication();

	TopLevel *toplevel = new TopLevel();
	toplevel->show();
	app->setMainWidget(toplevel);

	return app->exec();
}

