#ifndef QCANVAS_FAKE_H
#define QCANVAS_FAKE_H

typedef unsigned uint;

class QCanvasPixmapArray
{
	public:
		QCanvasPixmapArray();
		uint count() const {return (uint)framecount;}
	private:
		int framecount;
};

class QCanvasItem
{
	public:
		QCanvasItem();
		void hide();
		void show();
};

class QCanvasSprite : public QCanvasItem
{
	public:
		QCanvasSprite();
		void setSequence(QCanvasPixmapArray *a);
		void setFrame(int frame);
		void move(double dx, double dy);
		int frameCount() const {return images->count();}
		int frame() const {return frm;}
	private:
		int frm;
		QCanvasPixmapArray *images;
};

/* Fix for gcc 2.95.4 linker bug */
#if defined __GNUC__ && (__GNUC__ >= 3)
/* ok */
#else
class QTextStream
{
	public:
	QTextStream();
	int width(int foobar);
	//{return 0;}
	int precision(int barfoo);
	//{return 0;}
	int fill(int barfoo);
	//{return 0;}
};
#endif

#endif

