#ifndef SELECT_H
#define SELECT_H

#include <qdialog.h>
#include <qmap.h>

class QComboBox;
class PlayCanvas;
class Playground;
class QVBoxLayout;
class QFrame;
class QCheckBox;
class Rule;

class Select : public QDialog
{
	Q_OBJECT
	public:
		Select(QWidget *parent = NULL, const char *name = NULL);
		~Select();

		void setBots(int bots);
		void setBotsReadOnly();

		QString level();
		QString theme();
		int players();
		Rule *rules();

	private slots:
		void minimap();
		void slotAccept();
		void slotCustom();

	private:
		QMap<QString, QString> m_map, m_map2;
		QComboBox *combolevel, *combotheme, *comboplayers;
		PlayCanvas *m_playcanvas;
		Playground *m_playground;
		QVBoxLayout *xlayout;
		QFrame *preview;
		QComboBox *combocustom;
		Rule *m_rules;
};

#endif

