#include "blast.h"

Blast::Blast(int id, int x, int y, QCanvasSprite *sprite)
	: Sprite(id, x, y, sprite)
{
}

void Blast::proceed()
{
	if (((m_sprite) && (m_step < m_sprite->frameCount() - 1)) || ((!m_sprite) && (m_step < 5))) {
		if(m_sprite) m_sprite->setFrame(m_sprite->frame() + 1);
		m_step++;
	} else {
		if(m_sprite) m_sprite->hide();
		m_finished = 1;
	}
}

int Blast::step()
{
	return m_step;
}

