#ifndef _item_h
#define _item_h

#include "sprite.h"

class Item : public Sprite
{
public:
	Item(int id, int x, int y, QCanvasSprite *sprite, int type);

	virtual void proceed();
	int type();
	void setHighlighted(bool value);
	bool isHighlighted();
	void activate(bool value);
	bool isActive();

private:
	int m_type;
	bool m_highlighted;
	bool m_activated;
};

#endif
