#ifndef KAMIKAZE_EXIT_H
#define KAMIKAZE_EXIT_H

#include "sprite.h"

class Exit : public Sprite
{
public:
	Exit(int id, int x, int y, QCanvasSprite *sprite);

	virtual void proceed();
	void setHighlighted(bool value);
	bool isHighlighted();
	void activate(bool value);
	bool isActive();

private:
	bool m_activated;
	bool m_highlighted;
};

#endif

