#ifndef MNET_H
#define MNET_H

#include "net.h"
#include <list>

class MNet : public Net
{
	public:
		MNet();
		~MNet();

		enum ChannelOpcodes
		{
			add = -2001,
			remove = -2002,
			clear = -2003
		};

		enum ModeOpcodes
		{
			peer = -3001,
			multicast = -3002,
			broadcast = -3003
		};

		MNet &operator<<(ChannelOpcodes value);
		MNet &operator<<(ModeOpcodes value);
		MNet &operator<<(Net::SpecialOpcodes value);

		MNet &operator<<(int value);
		MNet &operator<<(const char* value);

	private:
		int m_mode;
		int waitforchannel;
		std::list<int> channellist;
		std::list<int>::iterator it;
};

#endif

