#ifndef SPLASH_H
#define SPLASH_H

#include <qwidget.h>

class Splash : public QWidget
{
	public:
		Splash(QWidget *parent = NULL, const char *name = NULL);
		~Splash();
		int autowidth();
		int autoheight();
};

#endif

