#ifndef RESOURCE_H
#define RESOURCE_H

#include <qstring.h>
#include <qpixmap.h>
#include <qcanvas.h>
#include <qmap.h>

class Resource
{
	public:
		enum ItemTypes
		{
			level,
			pic
		};

		static void setTheme(QString theme);
		static QString path(QString item);
		static QString load(QString item, int type);
		static QPixmap pixmap(QString item, int count);
		static QPixmap pixmapmulti(QString item, int num);
		static QCanvasPixmapArray *pixmaparray(QString item, int count);
		static int tilesize();
		static void setTilesize(int tilesize);
		static void clean();
		static QString theme();
		static void optimizeForSpeed(int optimize);

	private:
		static QMap<QString, QCanvasPixmapArray*> m_pixmaparrayhash;
		static QMap<QString, QPixmap*> m_pixmaphash;
};

#endif

