#ifndef KAMIKAZE_AI_PLAYER_H
#define KAMIKAZE_AI_PLAYER_H

#include "player.h"

class AI_Player : public Player
{
public:
	AI_Player(int id, int x, int y, QCanvasSprite* sprite);

	enum Direction { Up, Down, Left, Right };

	void direction(Direction direction);
	Direction direction(void);
	
	void counter(int counter) { m_counter = counter; }
	int counter(void) { return m_counter; }

	void clearWaypoints();
	void pushWaypoint(int x, int y);
	int nextX();
	int nextY();
	void popWaypoint();
	int numWaypoints();

private:
	Direction m_direction;	
	int m_counter;
	int *xwaypoints;
	int *ywaypoints;
	int m_waypoints;
	int m_firstwaypoint;
};

#endif

