#ifndef PLAYGROUND_H
#define PLAYGROUND_H

#include <qcanvas.h>
#include <qevent.h>

class Playground : public QCanvasView
{
	Q_OBJECT
	public:
		Playground(QCanvas *canvas = NULL, QWidget *parent = NULL, const char *name = NULL);
		~Playground();
		enum Keycodes
		{
			KeyUp,
			KeyDown,
			KeyLeft,
			KeyRight,
			KeyZoomIn,
			KeyZoomOut,
			KeyFullscreen,
		};
	signals:
		void key(int keycode);
	protected:
		void keyPressEvent(QKeyEvent *e);
};

#endif

