#ifndef _bomb_h
#define _bomb_h

#include "sprite.h"

class Player;

class Bomb : public Sprite
{
public:
	Bomb(int id, int x, int y, QCanvasSprite *sprite, Player *player);

	void proceed();
	int finished();

	void delay(bool delayed);

	void setMoveable(bool moveable);
	bool moveable();

	void setFlash(bool flash);
	bool flash();

	Player *player();

private:
	Player *m_player;
	bool m_delayed;
	bool m_moveable;
	bool m_flash;
};

#endif

