#include "qcanvas.h"

QCanvasPixmapArray::QCanvasPixmapArray()
{
	framecount = 0;
}

QCanvasSprite::QCanvasSprite()
: QCanvasItem()
{
	frm = 0;
	images = new QCanvasPixmapArray();
}

void QCanvasSprite::setSequence(QCanvasPixmapArray *a)
{
}

void QCanvasSprite::setFrame(int frame)
{
}

void QCanvasSprite::move(double dx, double dy)
{
}

QCanvasItem::QCanvasItem()
{
}

void QCanvasItem::hide()
{
}

void QCanvasItem::show()
{
}

/* Fix for gcc 2.95.4 linker bug */
#if defined __GNUC__ && (__GNUC__ >= 3)
/* ok */
#else
int QTextStream::fill(int foo)
{
	return 0;
}

int QTextStream::precision(int foo)
{
	return 0;
}

int QTextStream::width(int foo)
{
	return 0;
}
#endif

